

--Module from Michael Ebesn @ nova-fusion.com

camera = {}

function camera:new()

	local o = {}
	o.x = 0
	o.y = 0

	o.scaleX = 1
	o.scaleY = 1

	o.layers = {}
	
	self.__index = self
	
	return setmetatable(o, self)
end

function camera:addLayer(scale, func)
	
	table.insert(self.layers, {["scale"]=scale, ["draw"]=func})
	table.sort(self.layers, function(a, b) return a.scale < b.scale end)
	
end

function camera:addListLayer(scale, items)
	
	local l = List:new(items)
	self:addLayer(scale, function() l:draw() end)
	
end

function camera:set()
	love.graphics.push()
	love.graphics.scale(1/self.scaleX, 1/self.scaleY)
	
	--print("Translating by "..self.x.." , "..self.y)
	love.graphics.translate(-self.x, -self.y)
	
end

function camera:unset()
	love.graphics.pop()
end

function camera:move(dx, dy)
	
	if type(dx) == "table" then
		
		tempX = dx.x
		dy = dx.y
		dx = tempX
		
	end

	self.x = self.x + (dx or 0)
	self.y = self.y + (dy or 0)
end

function camera:scale(sx, sy)
	sx = sx or 1
	self.scaleX = self.scaleX * sx
	self.scaleY = self.scaleY * (sy or sx)
end

function camera:setPosition(x, y)
	if type(x) == "table" then
		
		y = x.y
		x = x.x
		
	end
	
	self.x = x or self.x
	self.y = y or self.y
end

function camera:setScale(sx, sy)
  self.scaleX = sx or self.scaleX
  self.scaleY = sy or self.scaleY
end

function camera:mousePosition()
  return love.mouse.getX() * self.scaleX + self.x, love.mouse.getY() * self.scaleY + self.y
end

function camera:draw()
	local oldX, oldY = self.x, self.y
	
	for _, v in ipairs(self.layers) do
		self.x = oldX * v.scale
		self.y = oldY * v.scale
		
		self:set()
		v.draw()
		self:unset()
	
	end
	
	self.x, self.y = oldX, oldY
	
end