local gui = libs["quickie"]
local control = libs["control"]

Bullet = {}

Bullet.__index = Bullet

function Bullet:new(pos, vel)
	local o = {}
	
	o.pos = pos
	
	o.speed = 500
	
	o.vel = vel:norm() * o.speed
	
	o.life = 1
	o.t = 0
	
	return setmetatable(o,self)
end

function Bullet:update(dt)
	
	self.pos = self.pos + (self.vel * dt)
	self.pos = posLoops(self.pos)
	
	self.t = self.t + dt
end

function Bullet:draw()
	
	love.graphics.point(self.pos.x, self.pos.y)
	
end

-- Fisheye?
--[[
        extern Image map;
        extern number fac = .5;
        uniform sampler2D tex0;
        vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
        {
            vec4 mapcol = Texel(map, texture_coords.xy);
            vec2 uv = texture_coords.xy + (.75 - mapcol.rg)/(8/fac)*ceil(mapcol.r + mapcol.g);
            vec3 col = texture2D(tex0,uv).xyz;
         return vec4(col, 1);
        }
    ]]
	
--[[
	//Dynamically coloured sprites
	extern vec4 replace1;
	extern vec4 replace2;
	
	extern vec4 baseCol;
	extern vec4 secondaryCol;
	
	
	
	vec4 effect(vec4 colour, Image texture, vec2 tex_coords, vec2 pixel_coords)
	{
		vec4 pixel = Texel(texture, tex_coords)
		
		//Replace with arrays
		//
		//e.g.
		//for (int i = 1; i < toReplace.length(); i++)
		//{
		//	replace = toReplace[i]
		//	if pixel == replace
		//	{
		//		return colours[i];
		//	}
		//}
		if (pixel == replace1)
		{
			return baseCol;
		}
		else if (pixel == replace2)
		{
			return secondaryCol;
		}
		
	}

--]]

GasBag =
{
	minSize = 18,
	maxSize = 30,
	minSpeed = 200,
	maxSpeed = 250
}

GasBag.__index = GasBag

function GasBag:getSpeed(size)
	size = size or self.radius
	return remap(size, self.minSize, self.maxSize, self.maxSpeed, self.minSpeed, true)
end


function GasBag:new(pos, size)
	
	local o = {}
	
	o.pos = pos
	o.radius = size
	
	-- Incorporate size into it, smaller size = faster
	o.vel = Vector2:rand() * self:getSpeed(size)
	
	return setmetatable(o, self)
	
end

function GasBag:updateSpeed()
	
	self.vel = self.vel:norm() * self:getSpeed()
	
end

function GasBag:update(dt)
	
	self.pos = self.pos + (self.vel * dt)
	self.pos = posLoops(self.pos)
	
end

function GasBag:draw()
	
	love.graphics.circle("line", self.pos.x, self.pos.y, self.radius,100)
	
end

Billboard = {}

Billboard.__index = Billboard

function Billboard:new(image, pos, rot, scale, mode)
	mode = mode or "centered"
	
	local o = {}
	
	o.image = image
	o.pos = pos
	o.rot = rot
	o.scale = scale
	o.mode = modes
	o.colour = {255,255,255,255}
	return setmetatable(o, self)
end

function Billboard:draw()
	local off = Vector2()
	if self.mode == "centered" then
		off = Vector2(self.image:getWidth(), self.image:getHeight())/2
	end
	love.graphics.setColor(unpack(self.colour))
	love.graphics.draw(self.image, self.pos.x, self.pos.y, self.rot, self.scale.x, self.scale.y, off.x, off.y)
	
end

player =
{
	vulnerable = true,
	lives = 3,
	
	radius = 17,
	pos = Vector2:new(150, 150),
	
	turnSpeed = 0.07,
	dir = -math.pi/2,
	origDir = -math.pi/2,
	thrust = 150,
	damping = 0.05,
	maxVel = 1000,
	vel = Vector2:new(),
	
	fireRate = 0.35,
	t = 0
}


game = {}

function game:load()
	
	control.bind("left", "k", {"left", "a"})
	control.bind("right", "k", {"right", "d"})
	control.bind("thrust", "k", {"up", "w"})
	control.bind("fire", "k", {" ", "lctrl", "rctrl"})
	
	self.bullets = List:new()
	self.gasbags = List:new()
	
	SCORE = 0
	
	self._scores =
	{
		[30]=10,
		[24]=50,
		[18]=100,
	}
	
	self.width = love.window.getWidth()
	self.height = love.window.getHeight()
	
	-- Gui
	self.thrustSlider = {value=0.5}
	self.dampingSlider = {value=0.5}
	minDistance = 50
	maxDistance = math.sqrt( (self.width*self.width) + (self.height*self.height) )/2
	
	--[[self.billboards = List:new()
	
	love.graphics.setDefaultFilter("nearest", "nearest")
	
	self.countdowns = {
		love.graphics.newImage("assets/img/countdown1.png"),
		love.graphics.newImage("assets/img/countdown2.png"),
		love.graphics.newImage("assets/img/countdown3.png"),
	}
	
	self.countdown = Billboard:new(self.countdown[3], self.center, 0, Vector2(1,1))
	self.billboards:add(self.countdown)
	]]
	
	self.countdown = {colour={255,255,255,0},text="3",size=50,pos=self.center}
	self.countdown.font=love.graphics.newFont("assets/fonts/GUBBLE_FILLED.ttf",self.countdown.size)
	
	self.background = {
		image = love.graphics.newImage("assets/img/jupiter_colourized.jpg"),
		zoom=1.0,
	}
	self.background.width = self.background.image:getWidth()
	self.background.height = self.background.image:getHeight()
	
	player_sprite = love.graphics.newImage("assets/img/ship_spritesheet.png")
	player_sprite:setFilter("nearest", "nearest")
	local anim8 = libs["anim8"]
	
	g = anim8.newGrid(32,40, player_sprite:getWidth(), player_sprite:getHeight())
	
	player.idle = anim8.newAnimation(g(3,1), 1, false)
	player.thrusting = anim8.newAnimation(g(1,1, 1,2, 2,1, 2,2), 0.2)
	
	player.state = "idle"
	
	--self:centerPlayer()
	
	self.level = 0
	
	-- Debug
	--------
	self.points = {}
	
	self.debugFont = love.graphics.newFont(11)
	
	self.TEST = 255
	
	--RegLerper(self, "TEST", 255, 0, 3, true)
	
end

function game:drawCountdown()
	
	love.graphics.setColor(unpack(self.countdown.colour))
	love.graphics.setFont(self.countdown.font)
	
	local w = self.countdown.font:getWidth(self.countdown.text)
	local h = self.countdown.font:getHeight()
	
	love.graphics.print(self.countdown.text, self.countdown.pos.x, self.countdown.pos.y, 0, 1, 1, w/2, h/2)
	
end

function game:centerPlayer()
	player.pos = self.center:clone()
end

function game:resetGame()
	SCORE = 0
	self:centerPlayer()
	player.lives = 3
	
	self.background.zoom = 1.0
	self.gasbags = List:new()
	self.bullets = List:new()
end	

function game:setUpLevel(numGasbags, difficulty)
	self.level = self.level + 1
	-- Disable controls
	player.vel = Vector2:new()
	control.disable()
	-- Lerp zoom and (maybe) position back to center?
	
	RegLerper(player, {"pos","x"}, player.pos.x, self.center.x, 3)
	RegLerper(player, {"pos","y"}, player.pos.y, self.center.y, 3)
	
	StartLerp(player, "dir", player.dir, player.origDir, 3)
	StartLerp(self.background, "zoom", self.background.zoom, self.background.zoom+0.1, 3)
	--Initiate sequence of countdown then setUpLevel
	
	local s = {}
	
	local t = 0
	
	for i = 3,1,-1 do
		
		f = function()
			print("Setting countdown to "..tostring(i))
			self.countdown.text = tostring(i)
			print("countdown.text = "..tostring(self.countdown.text))
			self.countdown.colour[4] = 255
			
			StartLerp(self.countdown.colour, 4, 255, 0, 1)
		end
		
		s[f] = t
		
		t = t + 1
		
	end
	
	s[function()
			control.enable()
			
			--Generation code
			
			
			numGasbags = numGasbags or self.level
			
			self.bullets = List:new()
			self.gasbags = List:new()
			
			
			
			for i = 1, numGasbags do
				pos = nil
				while not pos do
					pos = Vector2:rand() * math.random(minDistance, maxDistance)
					if pos.x <0 or pos.x > self.width or
					  pos.y < 0 or pos.y > self.height then
						--Position not valid!
						pos = nil
					end
				end
				self.gasbags:add(GasBag:new(pos, GasBag.maxSize))
		
	end
		end]=3
	
	Sequence(s):start()

	
	
end

function game:score(size)
	
	return self._scores[size]
	
end

function game:onStart()
	self.width = love.window.getWidth()
	self.height = love.window.getHeight()
	
	self.center = Vector2:new(self.width/2, self.height/2)
	self.countdown.pos = self.center
	love.graphics.setBackgroundColor(100,25,0)
	
	if self.level == 0 then
		self:resetGame()
		self:setUpLevel()
	end
end

function posLoops(pos, width, height)
	
	width = width or game.width
	height = height or game.height
	
	newpos = Vector2:new(pos.x, pos.y)
	
	if pos.x < 0 then
		newpos.x = width + pos.x
	elseif pos.x > width then
		newpos.x = 0 + pos.x-width
	end

	if pos.y < 0 then
		newpos.y = height + pos.y
	elseif pos.y > height then
		newpos.y = 0 + height - pos.y
	end
	
	return newpos
	
end

function circleCollision(a, b)
	
	local x = ((a.radius * b.pos.x) + (b.radius * a.pos.x)) / (a.radius + b.radius)
	local y = ((a.radius * b.pos.y) + (b.radius * a.pos.y)) / (a.radius + b.radius)
	
	return Vector2:new(x,y) -- + a.pos
	
end

function game:update(dt)
	
	self.bullets:update(dt)
	
	for _, bullet in ipairs(self.bullets.items) do
		if bullet.t >= bullet.life then
			self.bullets:remove(bullet)
		else
		
			for _, gasbag in ipairs(self.gasbags.items) do
				
				local dis = gasbag.pos - bullet.pos
				
				if dis:len() <= gasbag.radius then
					
					-- Splitting code here
					self.gasbags:remove(gasbag)
					
					SCORE = SCORE + self:score(gasbag.radius) 
					
					gasbag.radius = gasbag.radius - 6 
					
					if gasbag.radius >= gasbag.minSize then
						
						g1 = GasBag:new(gasbag.pos, gasbag.radius)
						self.gasbags:add(g1)
						g2 = GasBag:new(g1.pos, g1.radius, -g1.vel)
						self.gasbags:add(g2)
					else
						
						if #self.gasbags.items == 0 then
							self:setUpLevel()
						end
						
					end
					
					
					
					self.bullets:remove(bullet)
					break
				end
				
			end
			
		end
		
	end
	
	self.gasbags:update(dt)
	
	for i = 1, #self.gasbags.items do
		local a = self.gasbags.items[i]
		
		if not a then break end
		
		local dis = (player.pos - a.pos):len()
		
		if dis <= player.radius + a.radius then
			
			-- Damage the Player
			
			if player.vulnerable then
				player.lives = player.lives - 1
				
				player.vulnerable = false
				
				RegTimer(2, function() player.vulnerable = true; player.alpha=255;print("Player is vulnerable again") end)
				player.alpha = 155
				
				self:centerPlayer()
				
				player.vel = Vector2:new()
				
				--RegLerper(player, "alpha", 155, 255, 2, true)
			end
			
			
			if player.lives <= 0 then
				
				self.level = 0
				self:resetGame()
				EndState("game")
				
			end
			
		end
		--[[
		for j = i+1, #self.gasbags.items do
			local b = self.gasbags.items[j]
			
			if (a and b) and (a~=b) then
				
				local dis = (a.pos - b.pos):len()
				
				if dis <= a.radius + b.radius then
					-- There's a collision
					
					table.insert(self.points, circleCollision(a,b))
					
					a.vel = Vector2:new()
					b.vel = Vector2:new()
				end
			end
		end
		--]]
	end
	
	-- Player stuff
	----------------
	
	if player.t > 0 then
		player.t = math.max(player.t - dt, 0)
	end
		
		--[[
	changed = gui.Slider{info=self.thrustSlider, pos={10, 60}}
	
	if changed then
		 player.thrust = self.thrustSlider.value * 300
	end
	]]--
	
	if player.vel.x ~= 0 and player.vel.y ~= 0 then
		player.pos = player.pos + (player.vel * dt)
		player.pos = posLoops(player.pos)
	end
	-- Input
	if control.isDown("left") then
		player.dir = player.dir - player.turnSpeed
	end
	
	if control.isDown("right") then
		player.dir = player.dir + player.turnSpeed
	end
	
	while player.dir > math.pi * 2 or player.dir < -math.pi do
		
		if player.dir > math.pi * 2 then
			player.dir = player.dir - 2*math.pi
		elseif player.dir < -math.pi then
			player.dir = player.dir + 2*math.pi
		end
		
	end
	
	local dir = Vector2:new(math.cos(player.dir), math.sin(player.dir))
	
	if control.isDown("thrust") then
		if player.state == "idle" then
			player.state = "thrusting"
		end
		player.vel = player.vel + (dir * player.thrust * dt)
		if player.vel:len() > player.maxVel then
			player.vel = player.vel:norm() * player.maxVel
		end
	else
		if player.state == "thrusting" then
			player.state = "idle"
		end
	end
	
	if control.isDown("fire") then
		if player.t == 0 then
			self.bullets:add(Bullet:new(Vector2:new(player.pos.x, player.pos.y), dir))
			player.t = player.fireRate
		end
	end
	
	-- Damping
	
	--[[
	changed = gui.Slider{info=self.dampingSlider, pos={10,80}}
	
	if changed then
		player.damping = self.dampingSlider.value / 10
	end
	]]--
	
	player.vel = player.vel - (player.vel * player.damping * dt)
	delta = dt
	
	player[player.state]:update(dt)
end

function game:draw()
	love.graphics.setColor(255,255,255,150)
	love.graphics.draw(self.background.image, self.center.x, self.center.y, 0, self.background.zoom, nil, self.background.width/2, self.background.height/2)
	
	love.graphics.setColor(255,255,255)
	self.bullets:draw()
	self.gasbags:draw()
	
	-- Player stuff
	----------------
	--love.graphics.setColor(255,0,0)
	love.graphics.setPointSize(15)
	--love.graphics.point(player.pos.x, player.pos.y)
	
	love.graphics.setColor(255,255,255,player.alpha)
	local dir = Vector2:new(math.cos(player.dir), math.sin(player.dir)) * player.radius
	local scale = 1.5
	player[player.state]:draw(player_sprite, player.pos.x, player.pos.y, player.dir + math.pi/2,scale, scale, (32/2), (40/2))
	
	-- Gui
	--self.billboards:draw()
	
	love.graphics.setColor(255,255,255)
	love.graphics.print(SCORE, self.center.x, 0, 0, 1, 1, self.countdown.font:getWidth(tostring(SCORE))/2, 0)
	self:drawCountdown()
	gui.core.draw()
	
	--love.graphics.setColor(255,255,255)
	--love.graphics.point(self.center.x, self.center.y)
	
	-- Debug
	--[[
	love.graphics.setFont(self.debugFont)
	--love.graphics.line(player.pos.x, player.pos.y, player.pos.x + dir.x, player.pos.y + dir.y)
	
	--love.graphics.circle("line", player.pos.x, player.pos.y, player.radius, 50)
	
	love.graphics.setColor(0,255,self.TEST)
	
	love.graphics.print("FPS = "..1/delta, 0,0)
	love.graphics.print("#bullets = "..#self.bullets.items, 0,20)
	love.graphics.print("Lives = "..player.lives, 0, 40)
	--]]
end

function game:keypressed(key)
	
	if key == "escape" then
		EndState("pause")
	end
	
end

--[[
function game:mousepressed(x,y,button)
	
	if button == "l" then
		
		self.gasbags:add(GasBag:new(Vector2:new(x,y), math.random(GasBag.minSize, GasBag.maxSize)))
		
	end
	
end
--]]

RegState("game", game)
RegInitialState("game")